@@ -88,12 +88,22 @@ KisLiquifyTransformWorker::~KisLiquifyTransformWorker()
 
 bool KisLiquifyTransformWorker::operator==(const KisLiquifyTransformWorker &other) const
 {
-    return
-        m_d->srcBounds == other.m_d->srcBounds &&
-        m_d->originalPoints == other.m_d->originalPoints &&
-        m_d->transformedPoints == other.m_d->transformedPoints &&
-        m_d->pixelPrecision == other.m_d->pixelPrecision &&
-        m_d->gridSize == other.m_d->gridSize;
+    bool result =
+            m_d->srcBounds == other.m_d->srcBounds &&
+            m_d->pixelPrecision == other.m_d->pixelPrecision &&
+            m_d->gridSize == other.m_d->gridSize &&
+            m_d->originalPoints.size() == other.m_d->originalPoints.size() &&
+            m_d->transformedPoints.size() == other.m_d->transformedPoints.size();
+
+    if (!result) return false;
+
+    const qreal eps = 1e-6;
+
+    result =
+        KisAlgebra2D::fuzzyPointCompare(m_d->originalPoints, other.m_d->originalPoints, eps) &&
+        KisAlgebra2D::fuzzyPointCompare(m_d->transformedPoints, other.m_d->transformedPoints, eps);
+
+    return result;
 }
 
 int KisLiquifyTransformWorker::pointToIndex(const QPoint &cellPt)
@@ -442,11 +452,11 @@ QRect KisLiquifyTransformWorker::approxChangeRect(const QRect &rc)
     const int step = qMax(minStep, m_d->transformedPoints.size() / maxSamplePoints);
 
     QVector<QPoint> samplePoints;
-    for (int i = 0; i < m_d->transformedPoints.size(); i += step) {
-        samplePoints << m_d->transformedPoints[i].toPoint();
+    for (auto it = m_d->transformedPoints.constBegin(); it != m_d->transformedPoints.constEnd(); ++it) {
+        samplePoints << it->toPoint();
     }
 
-    QRect resultRect = KritaUtils::approximateRectFromPoints(samplePoints);
+    QRect resultRect = KisAlgebra2D::approximateRectFromPoints(samplePoints);
     return KisAlgebra2D::blowRect(resultRect | rc, margin);
 }
 
