@@ -35,6 +35,7 @@ public:
 
     void processDirectly(KisPaintDeviceSP src,
                          KisMultipleProjection *dst,
+                         KisLayerStyleKnockoutBlower *blower,
                          const QRect &applyRect,
                          KisPSDLayerStyleSP style,
                          KisLayerStyleFilterEnvironment *env) const override;
@@ -44,6 +45,14 @@ public:
 
 private:
     KisLsSatinFilter(const KisLsSatinFilter &rhs);
+
+
+    void applySatin(KisPaintDeviceSP srcDevice,
+                    KisMultipleProjection *dst,
+                    const QRect &applyRect,
+                    const psd_layer_effects_context *context,
+                    const psd_layer_effects_satin *config,
+                    KisLayerStyleFilterEnvironment *env) const;
 };
 
 #endif
