@@ -194,7 +194,7 @@ KoShapeManager* fetchShapeManagerFromNode(KisNodeSP node)
 }
 }
 
-KisCanvas2::KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResourceProvider *resourceManager, KisView *view, KoShapeControllerBase *sc)
+KisCanvas2::KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResourceProvider *resourceManager, KisMainWindow *mainWindow, KisView *view, KoShapeControllerBase *sc)
     : KoCanvasBase(sc, resourceManager)
     , m_d(new KisCanvas2Private(this, coordConverter, view, resourceManager))
 {
@@ -204,8 +204,8 @@ KisCanvas2::KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResource
      * light.
      */
     m_d->bootstrapLodBlocked = true;
-    connect(view->mainWindow(), SIGNAL(guiLoadingFinished()), SLOT(bootstrapFinished()));
-    connect(view->mainWindow(), SIGNAL(screenChanged()), SLOT(slotConfigChanged()));
+    connect(mainWindow, SIGNAL(guiLoadingFinished()), SLOT(bootstrapFinished()));
+    connect(mainWindow, SIGNAL(screenChanged()), SLOT(slotConfigChanged()));
 
     KisImageConfig config(false);
 
@@ -314,7 +314,7 @@ void KisCanvas2::setCanvasWidget(KisAbstractCanvasWidget *widget)
     m_d->canvasWidget = widget;
 
     // Either tmp was null or we are being called by KisCanvas2 constructor that is called by KisView
-    // constructor, so the view manager still doesn't exists.
+    // constructor, so the view manager still doesn't exist.
     if(m_d->canvasWidget != 0 && viewManager() != 0)
         viewManager()->inputManager()->addTrackedCanvas(this);
 
@@ -630,8 +630,8 @@ void KisCanvas2::resetCanvas(bool useOpenGL)
 void KisCanvas2::startUpdateInPatches(const QRect &imageRect)
 {
     /**
-     * We don't do patched loading for openGL canvas, becasue it loads
-     * the tiles, which are bascially "patches". Therefore, big chunks
+     * We don't do patched loading for openGL canvas, because it loads
+     * the tiles, which are basically "patches". Therefore, big chunks
      * of memory are never allocated.
      */
     if (m_d->currentCanvasIsOpenGL) {
@@ -1084,13 +1084,7 @@ void KisCanvas2::slotConfigChanged()
     //       the coordinates should be able to work around this.
     // FIXME: We should change to associate the display profiles with the screen
     //        model and serial number instead. See https://bugs.kde.org/show_bug.cgi?id=407498
-    QScreen *canvasScreen = this->canvasWidget()->window()->windowHandle()->screen();
-    QPoint canvasScreenCenter = canvasScreen->geometry().center();
-    int canvasScreenNumber = QApplication::desktop()->screenNumber(canvasScreenCenter);
-    if (canvasScreenNumber == -1) {
-        // Fall back to the old way of getting the screenNumber
-        canvasScreenNumber = QApplication::desktop()->screenNumber(this->canvasWidget());
-    }
+    int canvasScreenNumber = QApplication::desktop()->screenNumber(this->canvasWidget());
     if (canvasScreenNumber != -1) {
         setDisplayProfile(cfg.displayProfile(canvasScreenNumber));
     } else {
@@ -1255,8 +1249,6 @@ void KisCanvas2::setLodAllowedInCanvas(bool value)
 
     KisConfig cfg(false);
     cfg.setLevelOfDetailEnabled(m_d->lodAllowedInImage);
-
-    KisUsageLogger::log(QString("Instant Preview Setting: %1").arg(m_d->lodAllowedInImage));
 }
 
 bool KisCanvas2::lodAllowedInCanvas() const
