@@ -21,12 +21,14 @@
 #include <kis_assistant_tool.h>
 
 #include <QPainter>
+#include <QPainterPath>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 #include <QStandardPaths>
 #include <QFile>
 #include <QLineF>
 
+
 #include <kis_debug.h>
 #include <klocalizedstring.h>
 #include <KColorButton>
@@ -101,7 +103,7 @@ void KisAssistantTool::deactivate()
 void KisAssistantTool::beginPrimaryAction(KoPointerEvent *event)
 {
     setMode(KisTool::PAINT_MODE);
-    m_origAssistantList = cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants());
+    m_origAssistantList = KisPaintingAssistant::cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants());
 
     bool newAssistantAllowed = true;
 
@@ -525,7 +527,7 @@ void KisAssistantTool::endPrimaryAction(KoPointerEvent *event)
             m_assistantDrag.clear();
         }
         dbgUI << "creating undo command...";
-        KUndo2Command *command = new EditAssistantsCommand(m_canvas, m_origAssistantList, cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()));
+        KUndo2Command *command = new EditAssistantsCommand(m_canvas, m_origAssistantList, KisPaintingAssistant::cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()));
         m_canvas->viewManager()->undoAdapter()->addCommand(command);
         dbgUI << "done";
     } else if(m_internalMode == MODE_DRAGGING_TRANSLATING_TWONODES) {
@@ -539,18 +541,6 @@ void KisAssistantTool::endPrimaryAction(KoPointerEvent *event)
     m_canvas->updateCanvas(); // TODO update only the relevant part of the canvas
 }
 
-QList<KisPaintingAssistantSP> KisAssistantTool::cloneAssistantList(const QList<KisPaintingAssistantSP> &list) const
-{
-    QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> handleMap;
-    QList<KisPaintingAssistantSP> clonedList;
-    dbgUI << "cloning assistants...";
-    for (auto i = list.begin(); i != list.end(); ++i) {
-        clonedList << (*i)->clone(handleMap);
-    }
-    dbgUI << "done";
-    return clonedList;
-}
-
 void KisAssistantTool::addAssistant()
 {
     m_canvas->paintingAssistantsDecoration()->addAssistant(m_newAssistant);
@@ -561,7 +551,7 @@ void KisAssistantTool::addAssistant()
     }
 
     QList<KisPaintingAssistantSP> assistants = m_canvas->paintingAssistantsDecoration()->assistants();
-    KUndo2Command *addAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, cloneAssistantList(assistants), EditAssistantsCommand::ADD, assistants.indexOf(m_newAssistant));
+    KUndo2Command *addAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, KisPaintingAssistant::cloneAssistantList(assistants), EditAssistantsCommand::ADD, assistants.indexOf(m_newAssistant));
     m_canvas->viewManager()->undoAdapter()->addCommand(addAssistantCmd);
 
     m_handles = m_canvas->paintingAssistantsDecoration()->handles();
@@ -581,7 +571,7 @@ void KisAssistantTool::removeAssistant(KisPaintingAssistantSP assistant)
     }
     m_canvas->paintingAssistantsDecoration()->removeAssistant(assistant);
 
-    KUndo2Command *removeAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()), EditAssistantsCommand::REMOVE, assistants.indexOf(assistant));
+    KUndo2Command *removeAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, KisPaintingAssistant::cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()), EditAssistantsCommand::REMOVE, assistants.indexOf(assistant));
     m_canvas->viewManager()->undoAdapter()->addCommand(removeAssistantCmd);
 
     m_handles = m_canvas->paintingAssistantsDecoration()->handles();
@@ -613,8 +603,16 @@ void KisAssistantTool::updateToolOptionsUI()
          // load custom color settings from assistant (this happens when changing assistant
          m_options.useCustomAssistantColor->setChecked(m_selectedAssistant->useCustomColor());
          m_options.customAssistantColorButton->setColor(m_selectedAssistant->assistantCustomColor());
-         float opacity = (float)m_selectedAssistant->assistantCustomColor().alpha()/255.0 * 100.0 ;
-         m_options.customColorOpacitySlider->setValue(opacity);
+
+
+         double opacity = (double)m_selectedAssistant->assistantCustomColor().alpha()/(double)255.00 * (double)100.00 ;
+         opacity = ceil(opacity); // helps keep the 0-100% slider from shifting
+
+         m_options.customColorOpacitySlider->blockSignals(true);
+         m_options.customColorOpacitySlider->setValue((double)opacity);
+         m_options.customColorOpacitySlider->blockSignals(false);
+
+
      } else {
          m_options.vanishingPointAngleSpinbox->setVisible(false); //
      }
@@ -721,8 +719,14 @@ void KisAssistantTool::paint(QPainter& _gc, const KoViewConverter &_converter)
 
 void KisAssistantTool::removeAllAssistants()
 {
+    m_origAssistantList = m_canvas->paintingAssistantsDecoration()->assistants();
+
     m_canvas->viewManager()->canvasResourceProvider()->clearPerspectiveGrids();
     m_canvas->paintingAssistantsDecoration()->removeAll();
+
+    KUndo2Command *removeAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, KisPaintingAssistant::cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()));
+    m_canvas->viewManager()->undoAdapter()->addCommand(removeAssistantCmd);
+
     m_handles = m_canvas->paintingAssistantsDecoration()->handles();
     m_canvas->updateCanvas();
 
@@ -792,23 +796,24 @@ void KisAssistantTool::loadAssistants()
                     errors = true;
                 }
 
+                if (assistant) {
+                    // load custom shared assistant properties
+                    if (xml.attributes().hasAttribute("useCustomColor")) {
+                        QStringRef useCustomColor = xml.attributes().value("useCustomColor");
 
-               // load custom shared assistant properties
-               if ( xml.attributes().hasAttribute("useCustomColor")) {
-                   QStringRef useCustomColor = xml.attributes().value("useCustomColor");
+                        bool usingColor = false;
+                        if (useCustomColor.toString() == "1") {
+                            usingColor = true;
+                        }
+                        assistant->setUseCustomColor(usingColor);
+                    }
 
-                   bool usingColor = false;
-                   if (useCustomColor.toString() == "1") {
-                       usingColor = true;
-                   }
-                   assistant->setUseCustomColor(usingColor);
-               }
+                    if ( xml.attributes().hasAttribute("useCustomColor")) {
+                        QStringRef customColor = xml.attributes().value("customColor");
+                        assistant->setAssistantCustomColor( KisDomUtils::qStringToQColor(customColor.toString()) );
 
-               if ( xml.attributes().hasAttribute("useCustomColor")) {
-                   QStringRef customColor = xml.attributes().value("customColor");
-                   assistant->setAssistantCustomColor( KisDomUtils::qStringToQColor(customColor.toString()) );
-
-               }
+                    }
+                }
            }
 
             if (assistant) {
@@ -962,11 +967,8 @@ QWidget *KisAssistantTool::createOptionWidget()
         connect(m_options.assistantsColor, SIGNAL(changed(QColor)), SLOT(slotGlobalAssistantsColorChanged(QColor)));
         connect(m_options.assistantsGlobalOpacitySlider, SIGNAL(valueChanged(int)), SLOT(slotGlobalAssistantOpacityChanged()));
 
-
         connect(m_options.vanishingPointAngleSpinbox, SIGNAL(valueChanged(double)), this, SLOT(slotChangeVanishingPointAngle(double)));
 
-        //ENTER_FUNCTION() << ppVar(m_canvas) << ppVar(m_canvas && m_canvas->paintingAssistantsDecoration());
-
         // initialize UI elements with existing data if possible
         if (m_canvas && m_canvas->paintingAssistantsDecoration()) {
             const QColor color = m_canvas->paintingAssistantsDecoration()->globalAssistantsColor();
@@ -974,8 +976,6 @@ QWidget *KisAssistantTool::createOptionWidget()
             QColor opaqueColor = color;
             opaqueColor.setAlpha(255);
 
-            //ENTER_FUNCTION() << ppVar(opaqueColor);
-
             m_options.assistantsColor->setColor(opaqueColor);
             m_options.customAssistantColorButton->setColor(opaqueColor);
             m_options.assistantsGlobalOpacitySlider->setValue(color.alphaF() * 100.0);
@@ -1001,7 +1001,7 @@ QWidget *KisAssistantTool::createOptionWidget()
         m_options.vanishingPointAngleSpinbox->setPrefix(i18n("Density: "));
         m_options.vanishingPointAngleSpinbox->setSuffix(QChar(Qt::Key_degree));
         m_options.vanishingPointAngleSpinbox->setRange(1.0, 180.0);
-        m_options.vanishingPointAngleSpinbox->setSingleStep(0.5);
+        m_options.vanishingPointAngleSpinbox->setSingleStep(1.0);
 
 
         m_options.vanishingPointAngleSpinbox->setVisible(false);
