@@ -60,6 +60,7 @@ KisToolMultihand::KisToolMultihand(KoCanvasBase *canvas)
 
     m_helper =
         new KisToolMultihandHelper(paintingInformationBuilder(),
+                                   canvas->resourceManager(),
                                    kundo2_i18n("Multibrush Stroke"));
     resetHelper(m_helper);
     if (image()) {
@@ -171,13 +172,19 @@ void KisToolMultihand::paint(QPainter& gc, const KoViewConverter &converter)
         // add division guide lines if using multiple brushes
         if ((m_handsCount > 1 && m_transformMode == SYMMETRY) ||
             (m_handsCount > 1 && m_transformMode == SNOWFLAKE) ) {
-
-            qreal axesAngle = 360.0 / float(m_handsCount);
+            int axesCount;
+            if (m_transformMode == SYMMETRY){
+                 axesCount = m_handsCount;
+            }
+            else {
+                axesCount = m_handsCount*2;
+            }
+            qreal axesAngle = 360.0 / float(axesCount);
             float currentAngle = 0.0;
             float startingInsetLength = 20; // don't start each line at the origin so we can see better when all points converge
 
             // draw lines radiating from the origin
-            for( int i=0; i < m_handsCount; i++) {
+            for( int i=0; i < axesCount; i++) {
 
                 currentAngle = i*axesAngle;
 
@@ -431,7 +438,7 @@ QWidget* KisToolMultihand::createOptionWidget()
 
 
     customUI->axisRotationSpinbox->setSuffix(QChar(Qt::Key_degree));   // origin rotation
-    customUI->axisRotationSpinbox->setSingleStep(0.5);
+    customUI->axisRotationSpinbox->setSingleStep(1.0);
     customUI->axisRotationSpinbox->setRange(0.0, 90.0, 1);
     customUI->axisRotationSpinbox->setValue(m_configGroup.readEntry("axesAngle", 0.0));
     connect( customUI->axisRotationSpinbox, SIGNAL(valueChanged(qreal)),this, SLOT(slotSetAxesAngle(qreal)));
@@ -530,6 +537,7 @@ void KisToolMultihand::slotSetTransformMode(int index)
     customUI->subbrushLabel->setVisible(false);
     customUI->addSubbrushButton->setVisible(false);
     customUI->removeSubbrushButton->setVisible(false);
+    m_addSubbrushesMode = 0;
 
     // turn on what we need
     if (index == MIRROR) {
@@ -552,6 +560,7 @@ void KisToolMultihand::slotSetTransformMode(int index)
     else if (index == COPYTRANSLATE) {
         customUI->subbrushLabel->setVisible(true);
         customUI->addSubbrushButton->setVisible(true);
+        customUI->addSubbrushButton->setChecked(false);
         customUI->removeSubbrushButton->setVisible(true);
     }
 
